//***************************************************************
/// @file fli_pcie_priv.h
/// @brief Internal definition for the FLI Host Card Device Driver
/// @author      Tatakis
/// @date        
/// @copyright  
///
//
//
//***************************************************************

#ifndef FLIPCIE_PRIV_H_
#define FLIPCIE_PRIV_H_

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/pci.h>
#include <linux/init.h>
#include <linux/cdev.h>
#include <linux/uaccess.h>
#include <linux/types.h>
#include <linux/device.h>

#include "flipcie_regblocks.h"
#include "dma_engine_types.h"



/// @name Driver Version Constants
/// @{
#define FLI_PCIE_MAJOR_VERSION (1)
#define FLI_PCIE_MINOR_VERSION (2)
#define FLI_PCIE_BUILD_NUMBER  (1)
///@}

/// PCIe BAR specific data
typedef struct
{
   void *          hdl;   ///< handle to device memory
   unsigned long   start; ///< The start of device memory
   unsigned long   len;   ///< The size of device addr space
   unsigned long   flags;
} fli_bar_data_t;

// Data for the scatter-gather part of a DMA transaction
typedef struct
{
	unsigned long    ulMappingId;  // Id used to from user land to kernel land for streaming
	struct sg_table  sgTable;
	struct page      **pages;    // array of mem pages for the user buffer
	int              pages_got;  // number of pages returned by get_user_pages_fast()
	int              numEntries; // number of SG entries returned by pci_map_sg()
	bool             bSGAllocated;

	unsigned int     uiNumContigDescriptors;
	DMA_DESCRIPTOR_LIST *pContigDescList;  // Contiguous sg list to build Hardware descriptors from
	DMA_DESC_INFO    descInfo;             // The Hardware/FPGA descriptor info
	DMA_DESCRIPTOR   *pHWDescriptors;      // The actual HW descriptors

} fli_sg_mapping_t;

/// Device instance data
typedef struct
{
	int              major;         ///< device major number
	int              minor;         ///< device minor number
	fli_bar_data_t   bar0;          ///< Info on the PCI BAR for the general register addr space
	fli_bar_data_t   bar1;          ///< Info on the PCI BAR for the DMA register addr space

	// DMA Writeback and XDMA registers
	volatile XDMA_POLL_WB *pXDmaWBVirtAddr;
	dma_addr_t       uiPollPhysAddr;
	XDMA_REGS        XDmaRegs;

	// List of SG Mappings used for DMAinf image data
	struct mutex mutexDMAMapping;
	unsigned int uiNumStreamMappings;
	fli_sg_mapping_t *pStreamSgMapping;

	struct pci_dev *pci_devp;
	struct cdev        cdev;
} fli_cam_data_t;


// We are supporting up to 4 host cards in a box,
// we don't expect more than one
#define NUM_FLI_PCIE_DEVICE (4)

#define BAR_0        (0)       ///< Base address register 0
#define BAR_1        (1)       ///< Base address register 1
#define BAR_2        (2)       ///< Base address register 2- in case bar 1 is confused....


int  fli_cam_probe(struct pci_dev *pPciDev, struct class *fli_class, int idx, int major, int minor);
void fli_cam_remove(struct pci_dev *pPciDev, struct class *fli_class, int idx);
int  fli_cam_init(void);
void fli_cam_exit(void);

#endif /* FLIPCIE_PRIV_H_ */
